# MIT License
#
# Copyright (c) 2021 Bill.Yuan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import json

import pytest
import sys
import os
import io
import xml.etree.ElementTree as ET

sys.path.append("..")
from pyusb_chain.__main__ import USBDevicesChain
from pyusb_chain.usb_tree_view_tool import UsbTreeViewTool
from pyusb_chain.usb_device import USBDevice

CUR_PATH = os.path.dirname(os.path.abspath(__file__))


def test_commandline():
    sys.argv = ['.\\__main__.py', '-g', '-l', '-f', 'COM12', '-e', '-v debug']
    usbDevicesChain = USBDevicesChain()
    usbDevicesChain.command_process()
    assert usbDevicesChain.args.gui
    assert usbDevicesChain.args.list
    assert usbDevicesChain.args.filter == 'COM12'
    assert usbDevicesChain.args.export


def test_export_xml():
    tool = UsbTreeViewTool()
    exportedFile = tool.export_xml()
    assert os.path.exists(exportedFile) == True
    with io.open(exportedFile, 'r', encoding='utf8') as f:
        lines = f.readlines()
        assert len(lines) > 0
    os.remove(exportedFile)


def test_usb_tree_parse():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)
    assert len(tool.usbDevices) == 12


def test_filter_data():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)
    devices = tool.filter("COM16")
    assert len(devices) == 1

    devices = tool.filter("Audio")
    assert len(devices) == 3

    devices = tool.filter("CP2102")
    assert len(devices) == 2


def test_export_json():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)
    USBDevicesChain.export_json(tool.usbDevices)
    jsonData = None
    with io.open(USBDevicesChain.EXPORT_JSON_NAME, 'r', encoding='utf8') as f:
        jsonData = json.loads(f.read())
    os.remove(USBDevicesChain.EXPORT_JSON_NAME)
    assert len(jsonData) == 20


def test_usb_device_get_values_location_info():
    # base USB device
    values = USBDevice.get_values(
        "\r\nService : silabser\r\nEnumerator : USB\r\nLocation Info : Port_#0002.Hub_#0008\r\nLocation IDs: PCIROOT\r\n",
        r"\r\nLocation Info\s*:\s*.*?\r\n", ["Location Info", ":"])
    assert len(values) == 1
    assert values[0] == "Port_#0002.Hub_#0008"


def test_usb_device_get_values_device_id():
    values = USBDevice.get_values(
        "\r\nKernel Name: \\Device\\USBPDO-24\r\nDevice ID  : USB\\VID_10C4&PID_EA60\\EVKMIMXRT1170_1_A\r\nHardware IDs : USB\\VID_10C4&PID_EA60&REV_0100 USB\\VID_10C4&PID_EA60",
        r"\r\nDevice ID\s*:\s*.*?\r\n", ["Device ID", ":"])
    assert len(values) == 1
    assert values[0] == "USB\\VID_10C4&PID_EA60\\EVKMIMXRT1170_1_A"


def test_usb_device_get_values_sn():
    # base USB device SN
    values = USBDevice.get_values(
        'iProduct: 0x02 (String Descriptor 2)\r\n Language 0x0409 : "CP2102 USB to UART Bridge"\r\niSerialNumber: 0x03 (String Descriptor 3)\r\n Language 0x0409         : "evkmimxrt1170_1_a"\r\nbNumConfigurations       : 0x01 (1 Configuration)"',
        r"\r\niSerialNumber.*?\r\n Language 0x0409\s*:\s*.*?\r\n",
        ["iSerialNumber.*?\r\n", "Language 0x0409", ":", "\""])
    assert len(values) == 1
    assert values[0] == "evkmimxrt1170_1_a"


def test_usb_device_get_values_comport():
    # Com port
    values = USBDevice.get_values(
        'Power State : D0 (supported: D0, D2, D3, wake from D0, wake from D2)\r\nCOM-Port : COM16 (\\Device\\Silabser0)\r\n',
        r"COM-Port\s*:\s*.*?\(", ["COM-Port", ":", r"\("])
    assert len(values) == 1
    assert values[0] == "COM16"


def test_usb_device_get_values_multi_comports():
    # multi com ports
    values = USBDevice.get_values(
        'Power State : D0 (supported: D0, D2, D3, wake from D0, wake from D2)\r\nCOM-Port : COM16 (\\Device\\Silabser0)\r\nService : FTSER2K\r\n  COM-Port: COM10 (\\Device\\VCP1)',
        r"COM-Port\s*:\s*.*?\(", ["COM-Port", ":", r"\("])
    assert len(values) == 2
    assert values[0] == "COM16"
    assert values[1] == "COM10"


def test_usb_device_get_values_audio():
    # audio playback
    values = USBDevice.get_values(
        ' Child Device 1        : Speakers (USB Audio Device) (Audio Endpoint)\r\n  Device ID \r\n',
        r"Child Device 1\s*:\s*.*?\(Audio Endpoint\)", ["Child Device 1", ":", r"\(Audio Endpoint\)"])
    assert len(values) == 1
    assert values[0] == "Speakers (USB Audio Device)"

    # audio record
    values = USBDevice.get_values(
        ' Child Device 2        : Microphone (USB Audio Device) (Audio Endpoint)\r\n  Device ID \r\n',
        r"Child Device 2\s*:\s*.*?\(Audio Endpoint\)", ["Child Device 2", ":", r"\(Audio Endpoint\)"])
    assert len(values) == 1
    assert values[0] == "Microphone (USB Audio Device)"


def test_usb_device_get_values_driver_key():
    # driver key
    values = USBDevice.get_values(
        'PID_6001\r\nDriver KeyName: {36fc9e60-c465-11cf-8056-444553540000}\\0027 (GUID_DEVCLASS_USB)\r\nDriver',
        r"\r\nDriver KeyName\s*:\s*.*?\(", ["Driver KeyName", ":", r"\(", r"\{.*\}", r"\\"])
    assert len(values) == 1
    assert values[0] == "0027"
    assert int(values[0]) == 27


def test_usb_device_parse():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)

    # serial port CP2102
    usbPortDevice = tool.usbDevices[8]
    assert usbPortDevice.deviceName == "Silicon Labs CP2102 USB to UART Bridge Controller - COM16"
    assert usbPortDevice.portChain == "1-7-5"
    assert usbPortDevice.locInfo == "Port_#0005.Hub_#0004"
    assert usbPortDevice.deviceID == "USB\\VID_10C4&PID_EA60\\0001"
    assert usbPortDevice.sn is None
    assert usbPortDevice.get_com_port() == "COM16"

    # CMSIS-DAP serial port
    usbPortDevice = tool.usbDevices[9]
    assert usbPortDevice.deviceName == "ARM mbed Composite Device - F:\\, COM18, HID"
    assert usbPortDevice.portChain == "1-7-6"
    assert usbPortDevice.locInfo == "Port_#0006.Hub_#0004"
    assert usbPortDevice.deviceID == "USB\\VID_0D28&PID_0204\\0205000047784E4500349004D917002AE561000097969900"
    assert usbPortDevice.sn == "0205000047784e4500349004d917002ae561000097969900"
    assert usbPortDevice.get_com_port() == "COM18"

    # multi serial ports IMX8
    usbPortDevice = tool.usbDevices[0]
    assert usbPortDevice.deviceName == "Future Devices International FTDI Quad RS232-HS - COM9, COM10, COM11, COM12"
    assert usbPortDevice.portChain == "1-3-1"
    assert usbPortDevice.locInfo == "Port_#0001.Hub_#0002"
    assert usbPortDevice.deviceID == "USB\\VID_0403&PID_6011\\6&2ED78AA8&0&1"
    assert usbPortDevice.sn is None
    assert usbPortDevice.get_com_port() == "COM9"
    assert usbPortDevice.get_com_port(1) == "COM10"
    assert usbPortDevice.get_com_port(2) == "COM11"
    assert usbPortDevice.get_com_port(3) == "COM12"

    # USB audio
    usbPortDevice = tool.usbDevices[4]
    assert usbPortDevice.deviceName == "C-Media USB Audio Device - Audio, HID"
    assert usbPortDevice.portChain == "1-3-7-3"
    assert usbPortDevice.locInfo == "Port_#0003.Hub_#0003"
    assert usbPortDevice.deviceID == "USB\\VID_0D8C&PID_0014\\7&B60E087&0&3"
    assert usbPortDevice.sn is None
    assert usbPortDevice.audioPlaybackName == "Speakers (USB Audio Device)"
    assert usbPortDevice.audioRecordName == "Microphone (USB Audio Device)"

    usbPortDevice = tool.usbDevices[5]
    assert usbPortDevice.deviceName == "C-Media USB Audio Device - Audio, HID"
    assert usbPortDevice.portChain == "1-3-7-4"
    assert usbPortDevice.locInfo == "Port_#0004.Hub_#0003"
    assert usbPortDevice.deviceID == "USB\\VID_0D8C&PID_0014\\7&B60E087&0&4"
    assert usbPortDevice.sn is None
    assert usbPortDevice.audioPlaybackName == "Speakers (4- USB Audio Device)"
    assert usbPortDevice.audioRecordName == "Microphone (4- USB Audio Device)"


def test_usb_tree_view_tool_get_device():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)

    # test get from sn
    device = tool.get_from_sn("123456789")
    assert device is None
    device = tool.get_from_sn(None)
    assert device is None

    device = tool.get_from_sn("0205000047784e4500349004d917002ae561000097969900")
    assert device.sn == "0205000047784e4500349004d917002ae561000097969900"
    assert device.portChain == "1-7-6"

    # test get from chain
    device = tool.get_from_chain("1-3-4")
    assert device is None
    device = tool.get_from_chain(None)
    assert device is None

    device = tool.get_from_chain("1-7-6")
    assert device.portChain == "1-7-6"
    assert device.sn == "0205000047784e4500349004d917002ae561000097969900"

    device = tool.get_from_chain("1-3-1:0")
    assert device.portChain == "1-3-1"
    assert device.get_com_port(0) == "COM9"
    assert device.get_key("COM9") == "1-3-1:0"

    device = tool.get_from_chain("1-3-1:3")
    assert device.portChain == "1-3-1"
    assert device.get_com_port(3) == "COM12"
    assert device.get_key("COM12") == "1-3-1:3"

    device = tool.get_from_chain("1-3-7-4:Speaker")
    assert device.portChain == "1-3-7-4"
    assert device.get_key("Speakers (4- USB Audio Device)") == "1-3-7-4:Speaker"
    assert device.audioPlaybackName == "Speakers (4- USB Audio Device)"
    assert device.audioRecordName == "Microphone (4- USB Audio Device)"

    device = tool.get_from_chain("1-3-7-4:Microphone")
    assert device.portChain == "1-3-7-4"
    assert device.get_key("Microphone (4- USB Audio Device)") == "1-3-7-4:Microphone"
    assert device.audioPlaybackName == "Speakers (4- USB Audio Device)"
    assert device.audioRecordName == "Microphone (4- USB Audio Device)"

    # test get from port
    device = tool.get_from_port("COM121")
    assert device is None
    device = tool.get_from_port(None)
    assert device is None

    device = tool.get_from_port("COM12")
    assert device.portChain == "1-3-1"
    assert device.get_key("COM12") == "1-3-1:3"

    device = tool.get_from_port("Speakers (4- USB Audio Device)")
    assert device.portChain == "1-3-7-4"
    assert device.get_key("Speakers (4- USB Audio Device)") == "1-3-7-4:Speaker"
    assert device.audioPlaybackName == "Speakers (4- USB Audio Device)"
    assert device.audioRecordName == "Microphone (4- USB Audio Device)"


def test_usb_tree_view_tool_covert():
    exportXMLFile = os.path.join(CUR_PATH, "export_test.xml")
    tool = UsbTreeViewTool()
    tool.parse(exportXMLFile)

    assert tool.get_chain_from_port("COM16") == "1-7-5"
    assert tool.get_port_from_chain("1-7-5") == "COM16"
    assert tool.get_chain_from_port("COM18") == "1-7-6"
    assert tool.get_port_from_chain("1-7-6") == "COM18"
    assert tool.get_chain_from_port("COM9") == "1-3-1:0"
    assert tool.get_port_from_chain("1-3-1:0") == "COM9"
    assert tool.get_chain_from_port("COM9") == "1-3-1:0"
    assert tool.get_chain_from_port("COM12") == "1-3-1:3"
    assert tool.get_port_from_chain("1-3-1:3") == "COM12"
    assert tool.get_chain_from_port("Speakers (4- USB Audio Device)") == "1-3-7-4:Speaker"
    assert tool.get_port_from_chain("1-3-7-4:Speaker") == "Speakers (4- USB Audio Device)"
    assert tool.get_chain_from_port("Microphone (4- USB Audio Device)") == "1-3-7-4:Microphone"
    assert tool.get_port_from_chain("1-3-7-4:Microphone") == "Microphone (4- USB Audio Device)"



# MIT License
#
# Copyright (c) 2021 Bill.Yuan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import uuid
import logging
import re
import subprocess
import sys
import xml.etree.ElementTree as ET
from pyusb_chain.usb_device import USBDevice, AudioDevice, COMPortDevice

logger = logging.getLogger("pyusb_path")


class UsbTreeViewTool(object):
    """The USB Tree View Tool object implements the command line interface of free tool
    UsbTreeView.exe in Windows.
    To get the all connected USB devices information, including the USB port chain.
    """
    def __init__(self):
        self.currentPath = os.path.dirname(os.path.abspath(__file__))

        #: The UsbTreeView.exe location
        self.tool = os.path.join(self.currentPath, "UsbTreeView.exe")

        #: Store the scanned all connected USB devices (not including USB hubs)
        self.usbDevices = []

    def start_gui(self):
        """Start the UsbTreeView.exe directly
        :return: None
        """
        subprocess.Popen("\"{}\"".format(self.tool))

    def export_xml(self):
        """Use UsbTreeView.exe command line to export the XML format of all USB devices information in current PC.
        :return: the exported file name
        """
        randomUUID = uuid.uuid4()
        exportFile = "export_{}.xml".format(randomUUID)
        # export xml file
        cmd = "\"{} /X={}\"".format(self.tool, exportFile)
        os.system(cmd)
        return exportFile

    def scan(self):
        """First export the XML file, then parse the all scanned USB devices.
        The information will store in self.usbDevices.
        :return: None
        """
        print("Scanning all USB devices...")
        exportFile = None
        try:
            exportFile = self.export_xml()
            self.parse(exportFile)
        finally:
            # remove temp export file
            if os.path.exists(exportFile):
                os.remove(exportFile)

    def parse(self, exportFile):
        """Pasrse the XML file that exported by UsbTreeView.exe
        :param exportFile: the XML file that exported by UsbTreeView.exe
        :return: None
        """
        root = ET.parse(exportFile).getroot()
        for tag in root.iter('node'):
            name = tag.get('text')
            if ":" in name:
                usbHubReg = re.compile(r"Generic .* Hub")
                if usbHubReg.search(name):
                    continue
                info = tag[0].text
                usbSerialDeviceReg = re.compile(r"COM\d")
                usbAudioDeviceReg = re.compile("Audio")
                if usbSerialDeviceReg.search(name):
                    usbDevice = COMPortDevice(name, info)
                elif usbAudioDeviceReg.search(name):
                    usbDevice = AudioDevice(name, info)
                else:
                    usbDevice = USBDevice(name, info)
                usbDevice.parse()
                self.usbDevices.append(usbDevice)

    def get_from_sn(self, sn):
        """Get the usb device by the SN if the devcie has the SN.
        :param sn: the SN of the device to search
        :return: the UsbDevice (None if it's not found)
        """
        if not sn:
            return None

        snKey = sn.split(":")[0]
        for device in self.usbDevices:
            if snKey == device.sn:
                return device
        logger.warning("Cannot get USB device from sn: {}!".format(sn))
        return None

    def get_from_chain(self, chain):
        """Get the usb device by the chain.
        :param chain: the chain to search, like 1-2-3, or 1-5-4:Speaker, 1-5-4:Microphone for audio device,
                      if there are more than 1 com ports associated to the same chain, use index like: 1-3-5:0
        :return: the UsbDevice (None if it's not found)
        """
        if not chain:
            return None

        chainKey = chain.split(":")[0]
        for device in self.usbDevices:
            if chainKey == device.portChain:
                return device
        logger.warning("Cannot get USB device from chain: {}!".format(chain))
        return None

    def get_from_port(self, port):
        """Get the usb device by the port.
        :param port: the port name to search, like "COM17", "Speakers (4- USB Audio Device)"
        :return: the UsbDevice (None if it's not found)
        """
        if not port:
            return None

        for device in self.usbDevices:
            if isinstance(device, AudioDevice):
                if port == device.audioPlaybackName or port == device.audioRecordName:
                    return device
            elif isinstance(device, COMPortDevice):
                if port in device.comPorts:
                    return device
            elif isinstance(device, USBDevice):
                if port == device.deviceID:
                    return device
        logger.warning("Cannot get USB device from port: {}!".format(port))
        return None

    def get_chain_from_port(self, port):
        """Get the chain from the port name.
        :param port: the port name to search, like "COM17", "Speakers (4- USB Audio Device)"
        :return: the chain, like 1-2-3, or 1-5-4:Speaker, 1-5-4:Microphone for audio device,
                 if there are more than 1 com ports associated to the same chain, use index like: 1-3-5:0
        """
        device = self.get_from_port(port)
        if device:
            return device.get_key(port=port)
        return None

    def get_port_from_chain(self, chain=None):
        """Get the port name from the chain
        :param chain: the chain to search, like 1-2-3, or 1-5-4:Speaker, 1-5-4:Microphone for audio device,
                      if there are more than 1 com ports associated to the same chain, use index like: 1-3-5:0
        :return: the port name, like "COM17", "Speakers (4- USB Audio Device)"
        """
        device = self.get_from_chain(chain)
        if device:
            return device.get_port(chain)
        return None

    def filter(self, filters):
        """Filter the usb devices by keywords.
        :param filters: keywords to be search (not case sensitive), use ',' to separate multi-keys.
                        Once there is the key matched in all information of the usb device
                        (port chain, device name, sn, .etc), it will be included in the return list
        :return: the usb devices list
        """
        devices = []
        for device in self.usbDevices:
            dataHash = device.export_data()
            if filters and not UsbTreeViewTool.__include(dataHash, filters):
                continue
            devices.append(device)
        return devices

    @staticmethod
    def __include(data, filters):
        line = ""
        for item in data:
            if item:
                line = line + "{}".format(item)

        filterList = filters.split(",")
        for f in filterList:
            if f.lower() in line.lower():
                return True
        return False



# MIT License
#
# Copyright (c) 2021 Bill.Yuan
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import argparse
import logging
import sys
import io
import json
from sys import platform
from tabulate import tabulate
from pyusb_chain.usb_tree_view_tool import UsbTreeViewTool
from pyusb_chain._version import VERSION

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("pyusb_path")


class USBDevicesChain(object):
    """Command line interface to list or search all connected USB devices.
    It implements the table print and export json features.
    """
    EXPORT_JSON_NAME = "usb_port_chain_export.json"

    def __init__(self):
        self.args = None
        pass

    def command_process(self):
        """Process command line
        :return: None
        """
        parser = argparse.ArgumentParser(description="Command line for port path of USB devices "
                                                     "(COM ports / Audio Devices)\r\nVersion:{}".format(VERSION))
        if "win32" == platform:
            parser.add_argument("-g", "--gui", action="store_true", default=False, dest="gui",
                help="Launch GUI of USBTreeViewer.exe in Windows system")
        parser.add_argument("-l", "--list", action="store_true", default=False, dest="list",
            help="List all USB devices information for COM ports and USB Audio devices")
        parser.add_argument("-a", "--allinfo", action="store_true", default=False, dest="allinfo",
            help="List all information of USB device, include SN and driver key")
        parser.add_argument("-f", "--filter", action="store", dest="filter",
            help="filter the key words of USB devices information")
        parser.add_argument("-e", "--export", action="store_true", default=False, dest="export",
            help="export the json format with all connected USB devices information")
        parser.add_argument("-v", "--verbose", action="store", dest="verbose",
            help="verbose log mode, 'debug', 'fatal', 'error', 'warning', 'info'")

        self.args = parser.parse_args()
        # enable all info log first if there is -v
        if self.args.verbose:
            v = self.args.verbose.lower()
            if "debug" == v:
                logger.setLevel(logging.DEBUG)
            elif "fatal" == v:
                logger.setLevel(logging.FATAL)
            elif "error" == v:
                logger.setLevel(logging.ERROR)
            elif "warning" == v:
                logger.setLevel(logging.WARNING)
            elif "info" == v:
                logger.setLevel(logging.INFO)

        logger.debug(sys.argv)
        logger.debug(self.args)

        if not self.args.list and not self.args.filter and not self.args.export:
            if "win32" == platform:
                if not self.args.gui:
                    parser.print_help()
            else:
                parser.print_help()

    def process(self):
        """Process the action, start gui or list or export the json with filter options
        :return: None
        """
        tool = UsbTreeViewTool()
        if self.args.gui:
            tool.start_gui()
        elif self.args.list or self.args.filter or self.args.export:
            tool.scan()

        if self.args.list or self.args.filter:
            devices = tool.filter(self.args.filter)
            data = []
            headers = ["Port Chain Key", "Port Name", "Device Name"]
            if self.args.allinfo:
                headers.append("SN")
                headers.append("Driver Key")
            for device in devices:
                data = data + device.export_data(self.args.allinfo, jsonFormat=False)
            print("\r\n")
            print(tabulate(data, headers=headers))

        if self.args.export:
            devices = tool.filter(self.args.filter)
            USBDevicesChain.export_json(devices)

    @staticmethod
    def export_json(usbDevices):
        """Export json format file for information of usb devices
        :param usbDevices: the exported usb devices
        :return: None (a json file will be saved in user command line path)
        """
        data = {}
        try:
            for device in usbDevices:
                data.update(device.export_data(jsonFormat=True))
            with io.open(USBDevicesChain.EXPORT_JSON_NAME, 'w', encoding='utf-8') as fobj:
                if sys.version_info[0] <= 2:
                    fobj.write(unicode(json.dumps(data, ensure_ascii=False, indent=4)))
                else:
                    json.dump(data, fobj, indent=4)
            print("\nPlease get '{}' for dumped information!\n".format(USBDevicesChain.EXPORT_JSON_NAME))
        except Exception:
            logger.exception("Fail to save json file ''{}'".format(USBDevicesChain.EXPORT_JSON_NAME))


def main():
    usbDevicesChain = USBDevicesChain()
    usbDevicesChain.command_process()
    usbDevicesChain.process()


if __name__ == "__main__":
    main()
